==> 0.3.0&file=out_yasapi-0.3.0-src.tar.gz: out_yasapi-0.3.0/out_yasapi/yasapi_ring.c <==
/*
 * yasapi_ring.c
 * Copyright (C) 2015 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#include <stdint.h>

DWORD RingCreate(Ring *pRing, SIZE_T dwSize, int debug)
{
#if defined (YASAPI_DEBUG) // {
  pRing->debug=debug;
#endif // }

  // create ring buffer.
  pRing->pRep=(LPSTR)GlobalAlloc(
    GMEM_FIXED,     // _In_  UINT uFlags,
    dwSize          // _In_  SIZE_T dwBytes
  );

  if (NULL==pRing->pRep) {
    DMESSAGE(debug,"allocating ring buffer");
    goto rep;
  }
  
  pRing->pMax=pRing->pRep+dwSize;
  RingReset(pRing);
  pRing->Copy=RingCopyMemory;
  pRing->pClient=pRing;

  return dwSize;
// cleanup:
  GlobalFree(pRing->pRep);
rep:
  return -1;
}

void RingDestroy(Ring *pRing)
{
  GlobalFree(pRing->pRep);
}

void RingReset(Ring *pRing)
{
  pRing->pHead=pRing->pRep;
  pRing->pTail=pRing->pRep;
  pRing->dwWritten=0;
  pRing->dwAvailable=pRing->pMax-pRing->pRep;
}

SIZE_T RingWrite(Ring *pRing, LPSTR pData, SIZE_T dwSize)
{
  SIZE_T dwCopySize;

  if (dwSize>pRing->dwAvailable)
    dwSize=pRing->dwAvailable;

  if (0<dwSize) {
    if (0<(pRing->pHead-pRing->pTail)||dwSize<=(pRing->pMax-pRing->pTail)) {
      if (NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pTail,       // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwSize              // _In_  SIZE_T Length
        );
      }

      pRing->pTail+=dwSize;
    }
    else {
      dwCopySize=pRing->pMax-pRing->pTail;

      if (0<dwCopySize&&NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pTail,       // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );

        pData+=dwCopySize;
      }

      dwCopySize=dwSize-dwCopySize;

      if (0<dwCopySize&&NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pRep,        // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );
      }

      pRing->pTail=pRing->pRep+dwCopySize;
    }

    pRing->dwWritten+=dwSize;
    pRing->dwAvailable-=dwSize;
  }

  return dwSize;
}

SIZE_T RingRead(Ring *pRing, LPSTR pData, SIZE_T dwSize)
{
  SIZE_T dwCopySize;

  if (dwSize>pRing->dwWritten)
    dwSize=pRing->dwWritten;

  if (0<dwSize) {
    if (0<(pRing->pTail-pRing->pHead)||dwSize<=(pRing->pMax-pRing->pHead)) {
      if (NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pHead,       // _In_  const VOID *Source,
          dwSize              // _In_  SIZE_T Length
        );
      }

      pRing->pHead+=dwSize;
    }
    else {
      dwCopySize=pRing->pMax-pRing->pHead;

      if (0<dwCopySize&&NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pHead,       // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );

        pData+=dwCopySize;
      }

      dwCopySize=dwSize-dwCopySize;

      if (0<dwCopySize&&NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pRep,        // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );
      }

      pRing->pHead=pRing->pRep+dwCopySize;
    }

    pRing->dwAvailable+=dwSize;
    pRing->dwWritten-=dwSize;
  }

  return dwSize;
}

void RingCopyMemory(PVOID *Client, PVOID Destination, const VOID *Source,
    SIZE_T Length)
{
  CopyMemory(
    Destination,              // _In_  PVOID Destination,
    Source,                   // _In_  const VOID *Source,
    Length                    // _In_  SIZE_T Length
  );
}

==> 0.4.0&file=out_yasapi-0.4.0-src.tar.gz: out_yasapi-0.4.0/out_yasapi/yasapi_ring.c <==
/*
 * yasapi_ring.c
 * Copyright (C) 2015 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#include <stdint.h>

DWORD RingCreate(Ring *pRing, SIZE_T dwSize, int debug)
{
#if defined (YASAPI_DEBUG) // {
  pRing->debug=debug;
#endif // }

  // create ring buffer.
  pRing->pRep=(LPSTR)GlobalAlloc(
    GMEM_FIXED,     // _In_  UINT uFlags,
    dwSize          // _In_  SIZE_T dwBytes
  );

  if (NULL==pRing->pRep) {
    DMESSAGE(debug,"allocating ring buffer");
    goto rep;
  }
  
  pRing->pMax=pRing->pRep+dwSize;
  RingReset(pRing);
  pRing->Copy=RingCopyMemory;
  pRing->pClient=pRing;

  return dwSize;
// cleanup:
  GlobalFree(pRing->pRep);
rep:
  return -1;
}

void RingDestroy(Ring *pRing)
{
  GlobalFree(pRing->pRep);
}

void RingReset(Ring *pRing)
{
  pRing->pHead=pRing->pRep;
  pRing->pTail=pRing->pRep;
  pRing->dwWritten=0;
  pRing->dwAvailable=pRing->pMax-pRing->pRep;
}

SIZE_T RingWrite(Ring *pRing, LPSTR pData, SIZE_T dwSize, SIZE_T n)
{
  SIZE_T dwCopySize;

  if (dwSize>pRing->dwAvailable)
    dwSize=pRing->dwAvailable;

  if (0<dwSize) {
    if (0<(pRing->pHead-pRing->pTail)||dwSize<=(pRing->pMax-pRing->pTail)) {
      if (NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pTail,       // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwSize              // _In_  SIZE_T Length
        );
      }

      pRing->pTail+=dwSize;
    }
    else {
      dwCopySize=pRing->pMax-pRing->pTail;

      if (0<dwCopySize&&NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pTail,       // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );

        pData+=dwCopySize/n;
      }

      dwCopySize=dwSize-dwCopySize;

      if (0<dwCopySize&&NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pRep,        // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );
      }

      pRing->pTail=pRing->pRep+dwCopySize;
    }

    pRing->dwWritten+=dwSize;
    pRing->dwAvailable-=dwSize;
  }

  return dwSize;
}

SIZE_T RingRead(Ring *pRing, LPSTR pData, SIZE_T dwSize)
{
  SIZE_T dwCopySize;

  if (dwSize>pRing->dwWritten)
    dwSize=pRing->dwWritten;

  if (0<dwSize) {
    if (0<(pRing->pTail-pRing->pHead)||dwSize<=(pRing->pMax-pRing->pHead)) {
      if (NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pHead,       // _In_  const VOID *Source,
          dwSize              // _In_  SIZE_T Length
        );
      }

      pRing->pHead+=dwSize;
    }
    else {
      dwCopySize=pRing->pMax-pRing->pHead;

      if (0<dwCopySize&&NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pHead,       // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );

        pData+=dwCopySize;
      }

      dwCopySize=dwSize-dwCopySize;

      if (0<dwCopySize&&NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pRep,        // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );
      }

      pRing->pHead=pRing->pRep+dwCopySize;
    }

    pRing->dwAvailable+=dwSize;
    pRing->dwWritten-=dwSize;
  }

  return dwSize;
}

void RingCopyMemory(PVOID *Client, PVOID Destination, const VOID *Source,
    SIZE_T Length)
{
  CopyMemory(
    Destination,              // _In_  PVOID Destination,
    Source,                   // _In_  const VOID *Source,
    Length                    // _In_  SIZE_T Length
  );
}
